# Copyright (C) 2007-2010 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/

__all__ = ['get_context', 'join']


import os
import sys

import desktop


def endswith(x, suffices):
    for suffix in suffices:
        if x.endswith(suffix):
            return True
    return False


_CONTEXT = {}

# PLATFORM
if sys.platform.startswith('win'):
    _CONTEXT['platform'] = 'windows'
elif sys.platform.startswith('darwin'):
    _CONTEXT['platform'] = 'apple'
else:
    #assume linux
    _CONTEXT['platform'] = 'linux'

UNIX = _CONTEXT['platform'] in ('linux', 'apple')

# SETUP
if hasattr(sys, 'frozen'):
    _CONTEXT['distribute'] = 'frozen'
elif not endswith(sys.argv[0], ('.py', 'nosetests')):
    _CONTEXT['distribute'] = 'package'
else:
    _CONTEXT['distribute'] = 'source'

_CONTEXT['context'] = (_CONTEXT['platform'], _CONTEXT['distribute'])


class ContextError(Exception):
    pass


def join(*args):
    path = os.path.abspath(os.path.join(*args))
    if not os.path.exists(path):
        raise ContextError(path)
    return path


def join_create(*args):
    path = os.path.abspath(os.path.join(*args))
    if not os.path.exists(path):
        os.mkdir(path)
    return path


def _find_parent(path, base):
    if base == os.path.basename(path):
        return path
    if not path:
        raise ContextError('Can not find "%s" parent path.' % base)
    return _find_parent(os.path.abspath(os.path.join(path, '..')), base)


def _find_source_path(base_source):
    # __file__ may only be used to locate the source files
    return _find_parent(
        os.path.abspath(
            os.path.dirname(unicode(__file__, sys.getfilesystemencoding())),
        ),
        base_source,
    )


def get_context(app_name, base_source='source', sys_prefix=None):
    """Finds the right paths depending on the context, which consists of the
    platform and how it is distributed.

    context = (platform, distribute)
    platform = 'linux', 'apple' or 'windows'
    distribute = 'frozen', 'package' or 'source'
    """
    if sys_prefix is None:
        sys_prefix = sys.prefix
    context = _CONTEXT.copy()
    # initialize APP_NAME
    context['app_name'] = app_name
    # initialize APP_ROOT and DATA_PATH
    if context['distribute'] == 'source':
        # source: eg. trunk/source
        context['app_source_path'] = _find_source_path(base_source)
        # data: eg. trunk/data
        context['app_data_path'] = join(
            context['app_source_path'], '..', 'data')
        # doc: eg. trunk/doc
        context['app_doc_path'] = join(
            context['app_source_path'], '..', 'doc')
        # locale: eg. trunk/locale
        context['app_locale_path'] = join(
            context['app_source_path'], '..', 'locale')
    elif context['context'] == ('linux', 'package'):
        # source: eg. /usr/share/phatch/source
        context['app_source_path'] = _find_source_path(base_source)
        # data: eg. /usr/share/phatch/data
        context['app_data_path'] = join(sys_prefix, 'share', app_name, 'data')
        # doc: eg. /usr/share/doc/phatch/
        context['app_doc_path'] = join(sys_prefix, 'share', 'doc', app_name)
        # locale: eg. /usr/share/locale/
        context['app_locale_path'] = join(sys_prefix, 'share', 'locale')
    #elif context['context'] == ('apple', 'frozen'):
    #    # TODO: implement this!
    #    APP_ROOT = os.path.dirname(os.path.dirname(unicode(sys.executable,
    #        sys.getfilesystemencoding())))
    #    # eg. Phatch.app/Contents/Resources/
    #    context['app_data_path'] = join(APP_ROOT, 'Contents', 'Resources')
    #elif context['context'] == ('windows', 'frozen'):
    #    # The same as the exe dir
    #    APP_ROOT = os.path.dirname(unicode(sys.executable,
    #        sys.getfilesystemencoding()))
    #    context['app_data_path'] = os.path.join(APP_ROOT, 'data')
    else:
        raise ContextError(
            'Sorry your context%(context)s is not yet supported.' % context)

    # other data pats
    context['app_images_path'] = join(context['app_data_path'], 'images')

    # user paths
    context['desktop_path'] = desktop.DESKTOP_PATH
    context['user_path'] = desktop.USER_PATH
    if context['platform'] == 'linux':
        context['user_cache_path'] = join_create(desktop.USER_CACHE_PATH,
            app_name)
        context['user_config_path'] = join_create(desktop.USER_CONFIG_PATH,
            app_name)
        context['user_data_path'] = join_create(desktop.USER_DATA_PATH,
            app_name)
    else:
        dot = '.' + app_name
        context['user_cache_path'] = join_create(desktop.USER_PATH, dot,
            'cache')
        context['user_config_path'] = join_create(desktop.USER_PATH, dot,
            'config')
        context['user_data_path'] = join_create(desktop.USER_PATH, dot,
            'data')
    return context


if __name__ == '__main__':
    import pprint
    pprint.pprint(get_context('app', base_source='phatch'))
